@@ -39,7 +39,7 @@ class VariantPosition:
 
     fullmatch = re.compile(pos_with_groups, flags=re.ASCII).fullmatch
     """Callable[[str, int, int], Optional[Match[str]]]: fullmatch callable for parsing positions
-    
+
     Returns an :py:obj:`re.Match` object if the full string matches one of the position groups in :py:data:`pos_extended`.
     """
 
@@ -123,7 +123,10 @@ class VariantPosition:
         """
         if self.utr == other.utr:
             if self.position == other.position:
-                if self.intronic_position == other.intronic_position:
+                if (
+                    self.intronic_position == other.intronic_position
+                ):  # pragma: no cover
+                    # this case is covered by __eq__
                     return False
                 elif self.intronic_position is None:
                     return other.intronic_position > 0
