@@ -21,6 +21,11 @@ class TestCreateSingleVariantFromString(unittest.TestCase):
             "p.=",
             "c.=",
             "p.(=)",
+            "c.1_3=",
+            "c.12=",
+            "g.88_99=",
+            "c.43-6_595+12=",
+            "p.Glu27fs",
         ]
 
         invalid_variant_strings = [
@@ -33,12 +38,18 @@ class TestCreateSingleVariantFromString(unittest.TestCase):
             "a.78+5_78+10del",
             "77dup",
             "n.Pro12_Gly18dup",
+            "p.Pro12_Gly18insGlyProAla",
             "g.22_23insauc",
             "g.25_24del",
             "g.25_24ins",
+            "r.22_24insauc",
             "r.43-6_595+12delinsctt",
             "x.=",
             "c.(=)",
+            "p.(Gly24=)",
+            "p.Gly24(=)",
+            "p.Arg12LysfsTer18",
+            "p.Glu27fs*?",
         ]
 
         for s in valid_variant_strings:
@@ -63,6 +74,10 @@ class TestCreateSingleVariantFromString(unittest.TestCase):
             "p.=",
             "p.(=)",
             "n.=",
+            "c.1_3=",
+            "c.12=",
+            "g.88_99=",
+            "c.43-6_595+12=",
         ]
 
         for s in variant_strings:
@@ -70,6 +85,14 @@ class TestCreateSingleVariantFromString(unittest.TestCase):
                 v = Variant(s)
                 self.assertEqual(s, str(v))
 
+    def test_fs(self) -> None:
+        variant_strings = ["p.Glu27fs"]
+
+        for s in variant_strings:
+            with self.subTest(s=s):
+                v = Variant(s)
+                self.assertEqual(s, str(v))
+
     def test_del(self) -> None:
         variant_strings = [
             "g.44del",
@@ -131,23 +154,70 @@ class TestCreateSingleVariantFromString(unittest.TestCase):
                 self.assertEqual(s, str(v))
 
     def test_target_identical(self) -> None:
-        variant_strings = [f"{prefix}.=" for prefix in tuple("gmo" "cn" "r")]
+        identical_variant_strings = [
+            *[f"{prefix}.=" for prefix in tuple("gmo" "cn" "r")],
+            "p.(=)",
+            "c.1_3=",
+        ]
 
-        for s in variant_strings:
+        non_identical_variant_strings = [
+            "p.Ter345Lys",
+            "p.Cys22=",
+            "g.48C>A",
+            "c.122-6T>A",
+            "g.22delinsAACG",
+            "c.83_85delinsT",
+        ]
+
+        for s in identical_variant_strings:
             with self.subTest(s=s):
                 v = Variant(s)
                 self.assertTrue(v.is_target_identical())
 
-        for s in variant_strings:
+        for s in non_identical_variant_strings:
             with self.subTest(s=s):
                 v = Variant(s)
-                self.assertEqual(s, str(v))
+                self.assertFalse(v.is_target_identical())
+
+    def test_synonymous(self) -> None:
+        synonymous_variant_strings = ["p.Gly24=", "p.=", "p.(=)"]
+
+        nonsynonymous_variant_strings = ["p.Ter345Lys", "c.=", "g.48C>A"]
+
+        for s in synonymous_variant_strings:
+            with self.subTest(s=s):
+                v = Variant(s)
+                self.assertTrue(v.is_synonymous())
+
+        for s in nonsynonymous_variant_strings:
+            with self.subTest(s=s):
+                v = Variant(s)
+                self.assertFalse(v.is_synonymous())
+
+    def test_relaxed_ordering(self):
+        variant_tuples = [
+            ("c.78+10_78+5del", "c.78+5_78+10del"),
+            ("c.80_77dup", "c.77_80dup"),
+            ("p.Gly18_Pro12dup", "p.Pro12_Gly18dup"),
+            ("p.Pro13_Ala12insGlyProCys", "p.Ala12_Pro13insGlyProCys"),
+            ("r.23_22insauc", "r.22_23insauc"),
+            ("c.595+12_43-6delinsCTT", "c.43-6_595+12delinsCTT"),
+            ("p.Cys80_Ile71delinsSer", "p.Ile71_Cys80delinsSer"),
+            ("c.3_1=", "c.1_3="),
+            ("g.99_88=", "g.88_99="),
+            ("c.595+12_43-6=", "c.43-6_595+12="),
+        ]
+
+        for v, s in variant_tuples:
+            with self.subTest(v=v, s=s):
+                self.assertEqual(str(Variant(v, relaxed_ordering=True)), s)
 
 
 class TestCreateMultiVariantFromString(unittest.TestCase):
     def test_creation(self):
         variant_strings = [
             "p.[Glu27Trp;Ter345Lys]",
+            "p.[Glu27Trp;Lys212fs]",
             "p.[Gly18del;Glu27Trp;Ter345Lys]",
             "p.[Gln7_Asn19del;Glu27Trp;Ter345Lys]",
             "c.[1_35del;78+5_78+10del;122T>A]",
@@ -157,6 +227,9 @@ class TestCreateMultiVariantFromString(unittest.TestCase):
             "p.[Glu27Trp;=;Ter345Lys]",
             "p.[(=);Gly18del;Glu27Trp;Ter345Lys]",
             "c.[12T>A;=;78+5_78+10del]",
+            "c.[1_3=;12T>A;78+5_78+10del]",
+            "p.[Glu27fs;Arg48Lys]",
+            "p.[Glu27fs;Arg48fs]",
         ]
 
         for s in variant_strings:
@@ -198,6 +271,7 @@ class TestCreateMultiVariantFromString(unittest.TestCase):
             "p.[Pro27Trp;Glu27Tyr]",
             "p.[Gly18del;Gly18Tyr]",
             "p.[Gln7_Asn19del;Glu13Trp]",
+            "p.[Glu13Trp;Gln7_Asn19del]",
             "p.[Gln7_Asn19del;Glu13Trp;Ter345Lys]",
             "c.[1_95del;78+5_78+10del;122T>A]",
             "c.[1_95del;22T>A]",
@@ -211,6 +285,41 @@ class TestCreateMultiVariantFromString(unittest.TestCase):
 
 
 class TestCreateSingleVariantFromValues(unittest.TestCase):
+    def test_equal(self):
+        valid_dict_tuples = [
+            (
+                {
+                    "variant_type": "equal",
+                    "prefix": "p",
+                    "position": "27",
+                    "target": "Glu",
+                },
+                "p.Glu27=",
+            ),
+            (
+                {
+                    "variant_type": "equal",
+                    "prefix": "c",
+                    "start_position": "12",
+                    "end_position": "12",
+                },
+                "c.12=",
+            ),
+            (
+                {
+                    "variant_type": "equal",
+                    "prefix": "c",
+                    "start_position": "1",
+                    "end_position": "3",
+                },
+                "c.1_3=",
+            ),
+        ]
+
+        for d, s in valid_dict_tuples:
+            with self.subTest(d=d, s=s):
+                self.assertEqual(Variant(s), Variant(d))
+
     def test_sub(self):
         valid_dict_tuples = [
             (
@@ -239,6 +348,23 @@ class TestCreateSingleVariantFromValues(unittest.TestCase):
             with self.subTest(d=d, s=s):
                 self.assertEqual(Variant(s), Variant(d))
 
+    def test_fs(self):
+        valid_dict_tuples = [
+            (
+                {
+                    "variant_type": "fs",
+                    "prefix": "p",
+                    "position": 27,
+                    "target": "Glu",
+                },
+                "p.Glu27fs",
+            ),
+        ]
+
+        for d, s in valid_dict_tuples:
+            with self.subTest(d=d, s=s):
+                self.assertEqual(Variant(s), Variant(d))
+
     def test_ins(self):
         valid_dict_tuples = [
             (
@@ -353,6 +479,96 @@ class TestCreateSingleVariantFromValues(unittest.TestCase):
             with self.subTest(d=d, s=s):
                 self.assertEqual(Variant(s), Variant(d))
 
+    def test_extra_keys(self):
+        invalid_dicts = [
+            {
+                "variant_type": "sub",
+                "prefix": "p",
+                "position": 27,
+                "target": "Glu",
+                "variant": "Trp",
+                "bonus": "data",
+            },
+            {
+                "variant_type": "sub",
+                "prefix": "c",
+                "position": "122-6",
+                "start_target": "T",
+                "target": "T",
+                "variant": "A",
+            },
+            {
+                "variant_type": "delins",
+                "prefix": "p",
+                "start_target": "Ile",
+                "end_position": 80,
+                "end_target": "Cys",
+                "variant": "Ser",
+                "position": "Ala",
+            },
+        ]
+
+        for d in invalid_dicts:
+            with self.subTest(d=d):
+                with self.assertRaises(MaveHgvsParseError):
+                    Variant(d)
+
+    def test_missing_keys(self):
+        invalid_dicts = [
+            {"prefix": "p", "position": 27, "target": "Glu", "variant": "Trp"},
+            {"variant_type": "sub", "position": "122-6", "target": "T", "variant": "A"},
+            {
+                "variant_type": "delins",
+                "prefix": "p",
+                "start_target": "Ile",
+                "end_position": 80,
+                "end_target": "Cys",
+                "variant": "Ser",
+            },
+        ]
+
+        for d in invalid_dicts:
+            with self.subTest(d=d):
+                with self.assertRaises(MaveHgvsParseError):
+                    Variant(d)
+
+    def test_invalid_keys(self):
+        invalid_dicts = [
+            {
+                "variant_type": "equal",
+                "prefix": "p",
+                "start_position": "27",
+                "end_position": "27",
+                "target": "Glu",
+            },
+            {"variant_type": "dup", "prefix": "c", "position": 77},
+            {
+                "variant_type": "test",
+                "prefix": "c",
+                "start_position": 77,
+                "end_position": 77,
+            },
+            {
+                "variant_type": "fs",
+                "prefix": "c",
+                "position": "12",
+                "target": "T",
+            },
+        ]
+
+        for d in invalid_dicts:
+            with self.subTest(d=d):
+                with self.assertRaises(MaveHgvsParseError):
+                    Variant(d)
+
+    def test_invalid_type(self):
+        invalid_values = [1234, None, 5.55, ("p", "Ile", 80, "Cys")]
+
+        for v in invalid_values:
+            with self.subTest(v=v):
+                with self.assertRaises(ValueError):
+                    Variant(v)
+
 
 class TestCreateMultiVariantFromValues(unittest.TestCase):
     def test_create_multivariant(self):
@@ -397,29 +613,70 @@ class TestCreateMultiVariantFromValues(unittest.TestCase):
                 "c.[77dup;122-6T>A]",
             ),
         ]
+
+        invalid_dicts = [
+            [
+                {
+                    "variant_type": "sub",
+                    "position": 27,
+                    "target": "Glu",
+                    "variant": "Trp",
+                },
+                {
+                    "variant_type": "delins",
+                    "prefix": "p",
+                    "start_position": 71,
+                    "start_target": "Ile",
+                    "end_position": 80,
+                    "end_target": "Cys",
+                    "variant": "Ser",
+                },
+            ],
+            [
+                {
+                    "variant_type": "sub",
+                    "prefix": "p",
+                    "position": 27,
+                    "target": "Glu",
+                    "variant": "Trp",
+                },
+                {
+                    "variant_type": "sub",
+                    "prefix": "c",
+                    "position": "122-6",
+                    "target": "T",
+                    "variant": "A",
+                },
+            ],
+        ]
+
         for d, s in valid_dict_tuples:
             with self.subTest(d=d, s=s):
                 self.assertEqual(Variant(s), Variant(d))
 
+        for d in invalid_dicts:
+            with self.subTest(d=d):
+                with self.assertRaises(MaveHgvsParseError):
+                    Variant(d)
+
 
 class TestTargetSequenceValidation(unittest.TestCase):
-    def test_target_identity(self):
-        variant_tuples = [
-            ("ACGT", "g.="),
-            ("ACGT", "m.="),
-            ("ACGT", "o.="),
-            ("ACGT", "c.="),
-            ("ACGT", "n.="),
-            ("ACGU", "r.="),
-            ("RCQY", "p.="),
-            ("RCQY", "p.(=)"),
-        ]
+    def test_valid_dna_equal(self):
+        variant_tuples = [("ACGT", "c.1_2="), ("ACGT", "c.4="), ("ACGT", "c.=")]
 
         for target, s in variant_tuples:
             with self.subTest(target=target, s=s):
                 v = Variant(s, targetseq=target)
                 self.assertEqual(s, str(v))
 
+    def test_invalid_dna_equal(self):
+        variant_tuples = [("ACGT", "c.4_5="), ("ACGT", "c.10=")]
+
+        for target, s in variant_tuples:
+            with self.subTest(target=target, s=s):
+                with self.assertRaises(MaveHgvsParseError):
+                    Variant(s, targetseq=target)
+
     def test_matching_dna_substitution(self):
         variant_tuples = [
             ("ACGT", "c.1A>T"),
@@ -437,6 +694,7 @@ class TestTargetSequenceValidation(unittest.TestCase):
             ("ACGT", "c.1C>T"),
             ("ACGT", "c.3T>C"),
             ("ACGT", "c.[1A>T;3T>C]"),
+            ("ACGT", "c.5A>G"),
         ]
 
         for target, s in variant_tuples:
@@ -516,6 +774,22 @@ class TestTargetSequenceValidation(unittest.TestCase):
                 with self.assertRaises(MaveHgvsParseError):
                     Variant(s, targetseq=target)
 
+    def test_valid_protein_equal(self):
+        variant_tuples = [("RCQY", "p.Arg1="), ("RCQY", "p.Tyr4="), ("RCQY", "p.=")]
+
+        for target, s in variant_tuples:
+            with self.subTest(target=target, s=s):
+                v = Variant(s, targetseq=target)
+                self.assertEqual(s, str(v))
+
+    def test_invalid_protein_equal(self):
+        variant_tuples = [("RCQY", "p.Trp5=")]
+
+        for target, s in variant_tuples:
+            with self.subTest(target=target, s=s):
+                with self.assertRaises(MaveHgvsParseError):
+                    Variant(s, targetseq=target)
+
     def test_matching_protein_substitution(self):
         variant_tuples = [
             ("RCQY", "p.Arg1Ala"),
@@ -533,6 +807,7 @@ class TestTargetSequenceValidation(unittest.TestCase):
             ("RCQY", "p.Cys1Ala"),
             ("RCQY", "p.Ala3Trp"),
             ("RCQY", "p.[Arg1Ala;Cys3Trp]"),
+            ("RCQY", "p.Asp5Glu"),
         ]
 
         for target, s in variant_tuples:
@@ -540,6 +815,64 @@ class TestTargetSequenceValidation(unittest.TestCase):
                 with self.assertRaises(MaveHgvsParseError):
                     Variant(s, targetseq=target)
 
+    def test_matching_protein_fs(self):
+        variant_tuples = [
+            ("RCQY", "p.Arg1fs"),
+            ("RCQY", "p.Gln3fs"),
+        ]
+
+        for target, s in variant_tuples:
+            with self.subTest(target=target, s=s):
+                v = Variant(s, targetseq=target)
+                self.assertEqual(s, str(v))
+
+    def test_nonmatching_protein_fs(self):
+        variant_tuples = [
+            ("RCQY", "p.Cys1fs"),
+            ("RCQY", "p.Ala3fs"),
+            ("RCQY", "p.Asp5fs"),
+        ]
+
+        for target, s in variant_tuples:
+            with self.subTest(target=target, s=s):
+                with self.assertRaises(MaveHgvsParseError):
+                    Variant(s, targetseq=target)
+
+    def test_matching_protein_indel(self):
+        variant_tuples = [
+            ("RCQY", "p.Arg1del"),
+            ("RCQY", "p.Arg1_Gln3dup"),
+        ]
+
+        for target, s in variant_tuples:
+            with self.subTest(target=target, s=s):
+                v = Variant(s, targetseq=target)
+                self.assertEqual(s, str(v))
+
+    def test_nonmatching_protein_indel(self):
+        variant_tuples = [
+            ("RCQY", "p.Cys1del"),
+            ("RCQY", "p.Arg1_Asp3dup"),
+            ("RCQY", "p.Asp5del"),
+        ]
+
+        for target, s in variant_tuples:
+            with self.subTest(target=target, s=s):
+                with self.assertRaises(MaveHgvsParseError):
+                    Variant(s, targetseq=target)
+
+    def test_skips_extended(self):
+        variant_tuples = [
+            ("ACGT", "c.1+3A>T"),
+            ("ACGT", "c.*33G>C"),
+            ("ACGT", "c.43-6_595+12delinsCTT"),
+        ]
+
+        for target, s in variant_tuples:
+            with self.subTest(target=target, s=s):
+                v = Variant(s, targetseq=target)
+                self.assertEqual(s, str(v))
+
 
 class TestMiscMethods(unittest.TestCase):
     def test_is_multi_variant(self):
@@ -580,6 +913,8 @@ class TestMiscMethods(unittest.TestCase):
             "r.22g>u",
             "p.Ile71_Cys80delinsSer",
             "p.=",
+            "p.[Pro12_Gly18dup;Glu27Trp]",
+            "r.[22g>u;35del]",
         ]
 
         extended_variant_strings = [
@@ -588,6 +923,8 @@ class TestMiscMethods(unittest.TestCase):
             "c.43-6_595+12delinsCTT",
             "c.*33G>C",
             "r.33+12a>c",
+            "c.[12G>T;122-6T>A]",
+            "c.[43-6_595+12delinsCTT;*33G>C]",
         ]
 
         for s in non_extended_variant_strings:
@@ -615,6 +952,7 @@ class TestMiscProperties(unittest.TestCase):
         variant_tuples = [
             ("sub", "p.Glu27Trp"),
             ("sub", "c.122-6T>A"),
+            ("fs", "p.Glu27fs"),
             ("del", "g.44del"),
             ("del", "c.78+5_78+10del"),
             ("dup", "c.77dup"),
@@ -633,6 +971,7 @@ class TestMiscProperties(unittest.TestCase):
     def test_position(self):
         variant_tuples = [
             (VariantPosition("Glu27"), "p.Glu27Trp"),
+            (VariantPosition("Glu27"), "p.Glu27fs"),
             (VariantPosition("122-6"), "c.122-6T>A"),
             (VariantPosition("44"), "g.44del"),
             ((VariantPosition("78+5"), VariantPosition("78+10")), "c.78+5_78+10del"),
@@ -672,6 +1011,7 @@ class TestMiscProperties(unittest.TestCase):
         variant_tuples = [
             (("Glu", "Trp"), "p.Glu27Trp"),
             (("T", "A"), "c.122-6T>A"),
+            (None, "p.Glu27fs"),
             (None, "g.44del"),
             (None, "c.78+5_78+10del"),
             (None, "c.77dup"),
@@ -702,6 +1042,11 @@ class TestMiscProperties(unittest.TestCase):
                 v = Variant(s)
                 self.assertEqual(t, v.target_id)
 
+        for _, s in variant_tuples:
+            with self.subTest(s=s):
+                v = Variant(s)
+                self.assertEqual(s, str(v))
+
 
 if __name__ == "__main__":
     unittest.main()
